Package: ReDaMoR
Type: Package
Title: Relational Data Modeler
Version: 1.0.0
Authors@R: c(
    person(
        "Patrice", "Godard",
        role = c("aut", "cre","cph"),
        email = "patrice.godard@gmail.com"
    ),
    person(
        "Kai", "Lin",
        role = c("ctb")
    )
    )
Maintainer: Patrice Godard <patrice.godard@gmail.com>
Description: The aim of this package is to manipulate relational
   data models in R.
   It provides functions to create, modify and export data models
   in json format.
   It also allows importing models created
   with 'MySQL Workbench' (<https://www.mysql.com/products/workbench/>).
   These functions are accessible through a graphical user
   interface made with 'shiny'.
   Constraints such as types, keys, uniqueness and mandatory fields are
   automatically checked and corrected when editing a model.
   Finally, real data can be confronted to a model to check their compatibility.
URL: https://patzaw.github.io/ReDaMoR/,
        https://github.com/patzaw/ReDaMoR/
BugReports: https://github.com/patzaw/ReDaMoR/issues
Depends: R (>= 4.1), dplyr, visNetwork
Imports: readr, shiny, shinyjs, jsonlite, DT, colourpicker, rintrojs,
        markdown, rstudioapi, crayon, utils, graphics, stats, Matrix
Suggests: knitr, rmarkdown, igraph
License: GPL-3
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 15:14:50 UTC; pgodard
Author: Patrice Godard [aut, cre, cph],
  Kai Lin [ctb]
Repository: CRAN
Date/Publication: 2026-05-19 05:10:30 UTC
Built: R 4.6.0; ; 2026-05-19 07:09:32 UTC; unix
