# Generated by roxygen2: do not edit by hand

S3method("[",RelDataModel)
S3method("[<-",RelDataModel)
S3method("[[<-",RelDataModel)
S3method("names<-",RelDataModel)
S3method(c,RelDataModel)
S3method(format,RelTableModel)
S3method(get_foreign_keys,RelDataModel)
S3method(get_foreign_keys,RelTableModel)
S3method(length,RelDataModel)
S3method(length,RelTableModel)
S3method(lengths,RelTableModel)
S3method(lengths,default)
S3method(names,RelDataModel)
S3method(plot,RelDataModel)
S3method(print,RelTableModel)
export(RelDataModel)
export(RelTableModel)
export(SUPPTYPES)
export(add_field)
export(add_foreign_key)
export(add_index)
export(add_table)
export(as_type)
export(auto_layout)
export(check_foreign_keys)
export(check_types)
export(clean_autosaved_RelDataModels)
export(col_types)
export(confront_data)
export(confront_table_data)
export(conv_type_ref)
export(copy_fields)
export(correct_constraints)
export(df_to_model)
export(format_confrontation_report)
export(format_confrontation_report_md)
export(fromDBM)
export(get_foreign_keys)
export(guess_constraints)
export(identical_RelDataModel)
export(identical_RelTableModel)
export(index_table)
export(is.MatrixModel)
export(is.RelDataModel)
export(is.RelTableModel)
export(is_MM)
export(lengths)
export(list_autosaved_RelDataModel)
export(list_type_ref)
export(model_relational_data)
export(norm_type_ref)
export(order_fields)
export(order_indexes)
export(readSQLDataModel)
export(read_SQL_data_model)
export(read_json_data_model)
export(read_named_MM)
export(read_named_MM_header)
export(recover_RelDataModel)
export(remove_field)
export(remove_foreign_key)
export(remove_index)
export(remove_table)
export(rename_field)
export(rename_table)
export(set_primary_key)
export(set_unique_index)
export(toDBM)
export(update_field)
export(update_foreign_key)
export(update_table_display)
export(view_confrontation_report)
export(write_json_data_model)
import(dplyr)
import(readr)
import(rintrojs)
import(shiny)
import(visNetwork)
importFrom(DT,DTOutput)
importFrom(DT,dataTableProxy)
importFrom(DT,datatable)
importFrom(DT,formatStyle)
importFrom(DT,renderDT)
importFrom(DT,replaceData)
importFrom(Matrix,sparseMatrix)
importFrom(colourpicker,colourInput)
importFrom(crayon,bgGreen)
importFrom(crayon,bgMagenta)
importFrom(crayon,bgRed)
importFrom(crayon,bold)
importFrom(crayon,green)
importFrom(crayon,underline)
importFrom(crayon,white)
importFrom(crayon,yellow)
importFrom(graphics,plot)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,write_json)
importFrom(markdown,renderMarkdown)
importFrom(rstudioapi,viewer)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(shinyjs,useShinyjs)
importFrom(stats,complete.cases)
importFrom(stats,rbeta)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(utils,object.size)
importFrom(utils,packageName)
