% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TYPEDEF.R
\name{list_type_ref}
\alias{list_type_ref}
\title{List supported types references}
\usage{
list_type_ref()
}
\description{
List supported types references
}
