% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RelDataModel.R
\name{RelDataModel}
\alias{RelDataModel}
\title{Create a RelDataModel object}
\usage{
RelDataModel(l, checkFK = TRUE, createFKIndex = FALSE)
}
\arguments{
\item{l}{the list of table models (\link{RelTableModel} objects)}

\item{checkFK}{a logical indicating if foreign keys should be checked
(default: TRUE)}

\item{createFKIndex}{should an index be create for the foreign keys
(default: FALSE)}
}
\value{
A RelDataModel object.
}
\description{
Create a RelDataModel object
}
