% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RelDataModel.R
\name{remove_index}
\alias{remove_index}
\title{Remove an index from a table in a \link{RelDataModel}}
\usage{
remove_index(x, tableName, fieldNames)
}
\arguments{
\item{x}{a \link{RelDataModel}}

\item{tableName}{the name of the table to modify (a single character)}

\item{fieldNames}{the names of the fields composing the index}
}
\value{
A \link{RelDataModel}
}
\description{
Remove an index from a table in a \link{RelDataModel}
}
