% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TYPEDEF.R
\name{SUPPTYPES}
\alias{SUPPTYPES}
\title{Supported R types}
\usage{
SUPPTYPES
}
\description{
Supported R types
}
